#!/usr/bin/perl

#---------------------------------------------
# (c) Zhuhan Jiang
#     version 1.13  12 July 2007 (renamed version 2.01)
# Freeware under GNU copyright convention.
#---------------------------------------------

######################
# configuration
######################

# external configuration file
$externConfig="_conf-b.rc";

# include configuration file
do "$externConfig";

# fetch unit/purpose/duedates info
@cfgarray=split(/\n/,$cfgtext);


sub trim {
my($s)=@_;
  if($s=~/^\s*(.*\S)\s*$/) { return "$1"; }
  return "$s";
}

$defaultUnitCount=0;   # remember the selection
$defaultUnitName="";

sub parseCfg { # global: $defaultUnitCount, $defaultUnitName
my(@cfg)=@_;
my($data)=[];
my($count,$subcount)=(0,0);
my($title,$alias)=("","");
my($aline,$name,$due,$late);
my($entry)=[];
my($titlecount)=-1;
  foreach $aline(@cfg) {
    $name=$dir=$due=$late="";
    if($aline=~/^([^#]*)#.*$/) {
      $aline="$1";
    }
    if($aline=~/^\s*$/) { next; }
    if($aline=~/^(.*)$/) { $aline="$1"; }
    $aline=~tr/a-zA-Z0-9[](){}<>+\-*\/,.:!?@#$%^&= /_/cs;
    if($aline=~/^([^,]*),(.*)$/) {
      $name="$1"; $aline="$2";
      if($aline=~/^([^,]*),(.*)$/) {
        $dir="$1";  $aline="$2";
        if($aline=~/^([^,]*),(.*)$/) {
          $due="$1";  $aline="$2";
          if($aline=~/^([^,]*),(.*)$/) {
            $late="$1";  $aline="$2";
          } else { $late=$aline; }
        } else { $due=$aline; }
      } else { $dir=$aline; }
    } else { $name=$aline; }
    $name=trim($name);
    $dir=trim($dir);
    $due=trim($due);
    $late=trim($late);
    if($name=~/^\*+\s*(.*)$/) {
      $aline="$1";
      $titlecount++;
      if($name=~/^\*\*/) {
        $defaultUnitCount=$titlecount;
        $defaultUnitName=$aline;
      }
      # $name="$1";
      $name=$aline;
      if($title ne "") {
        $data->[$count++]=[$title,$alias,$entry];
      }
      $title=$name; $alias=$dir;
      $subcount=0;
      $entry=[];
    }
    else {
      $entry->[$subcount++]=[$name,$dir,$due,$late];
    }
  }
  if($#cfg>=0) { $data->[$count++]=[$title,$alias,$entry]; }
  return $data;
}


sub cvtCfg{ # global: $defaultUnitCount
my($data,$unit)=@_;
my($i,$j,$name,$entry);
my(%unitmap)=();
my(%purposemap)=();
my(%duedate)=();
my(%latedate)=();
  $i=0;
  if(!defined($unit)) { $unit=""; }
  while(1) {
    $entry=$data->[$i];
    if(!defined($entry)) { last; }
    $name=$entry->[0];
    $unitmap{"$name"}=$entry->[1];
    if("$name" eq "$unit" || $i==$defaultUnitCount && "$unit" eq "") {
      $entry=$entry->[2];
      $j=0;
      while(1) {
        if(!defined($entry->[$j])) { last; }
        $name=$entry->[$j][0];
        $purposemap{"$name"}=$entry->[$j][1];
        $duedate{"$name"}=$entry->[$j][2];
        $latedate{"$name"}=$entry->[$j][3];
        $j++;
      }
    }
    $i++;
  }
  return (\%unitmap,\%purposemap,\%duedate,\%latedate);
}


# fetch all the unit info
@cfgParsed=parseCfg(@cfgarray);
if(param('unitname') eq "") {
  param(-name=>'unitname',-default=>$defaultUnitName);
}
(*unitmap,*purposemap,*duedate,*latedate)=cvtCfg(@cfgParsed,param('unitname'));

# link password file with unitname via %unitmap;
#$passwordfile=$baseDir.$unitmap{ param('unitname') }.$fileForPasswords;

$unitShortName=$unitmap{ param('unitname') };
$passwordfile=$baseDir.$unitShortName.$fileForPasswords;


########
# submission essentials below

use strict 'refs';
use lib '..';
use CGI qw(:standard);
use CGI::Carp qw/fatalsToBrowser/;
use LWP::UserAgent;

# URL of this submission program
$submitURL=url();


sub codePassword {
my($rawpasswd)=@_;
my($a,$b,$c,$d,$i);
my(@num)=();
  for($i=0;$i<8;$i++) { $num[$i]=ord(substr("$rawpasswd",$i,1)); }
  $a=chr($num[0]);
  $b=chr($num[1]);
  for($i=1;$i<8;$i++) {
    if($num[$i]==32) {
      $num[$i]=$i+97;
      if($i==1) { $b="B"; }
    }
  }
  $c=$d=0;
  for($i=0;$i<4;$i++) {
    $c+=$num[2*$i];
    $d+=$num[2*$i+1];
  }
  return $a.$c.$b.$d;
}


sub checkPassword {
my($uid,$rawpasswd,$pfile)=@_;
my($passwd)=codePassword($rawpasswd);
my($aline,$ppasswd,$psubdir,$puid);
my($matched)=0;
  if(open(PASSFILE, "<$pfile")) {
    while(<PASSFILE>) {
      $aline=$_;
      ($ppasswd,$psubdir,$puid) = split(/,/, $aline);
      if($puid=~ /^(.*)\n$/) { $puid="$1"; }
      if($ppasswd eq $passwd  && $puid eq $uid) {
        $matched=1;
        last;
      }
    }
    close(PASSFILE);
  }
  return $matched;
}


sub checkCryptPassword {
my($uid,$rawpasswd,$pfile)=@_;
my($aline,$ppasswd,$psubdir,$puid);
my($matched)=0;
  if(open(CRYPTPASSFILE, "<$pfile")) {
    while(<CRYPTPASSFILE>) {
      $aline=$_;
      ($puid,$ppasswd,$psubdir) = split(/,/, $aline);
      if($psubdir=~ /^(.*)\n$/) { $psubdir="$1"; }
      if("$puid" eq "$uid" && crypt($rawpasswd,$ppasswd) eq "$ppasswd") {
        $matched=1;
        last;
      }
    }
    close(CRYPTPASSFILE);
  }
  return $matched;
}


sub passwordOk { # global: $passwordfile
my($uid,$password,$useCrypt)=@_;
  return $useCrypt?
    checkCryptPassword($uid,$password,$passwordfile)
    : checkPassword($uid,$password,$passwordfile);
}


sub submitPage { # $defaultUnitCount: global variable
my($msg)=param('msg');
my($key,$count);
my($title)=param('title');

  if($title=~/^\s*$/) { $title="LOGON FIRST"; }

  print header()
   . "<body><form name=SUBMIT action=\"$submitURL\" method=post ENCTYPE=\"multipart/form-data\">\n"
   . "<table cellpadding=30 bgcolor=" . $mainBackgroundColor . "><tr><td>\n"
   . "<input type=hidden name=mode value=''>\n"
   . "<table border=0>\n"
   . "<tr><td colspan=2 align=center><font face='Arial, Helvetica, sans-serif' size=+3><b>$title</b></font></td></tr>\n"
   . "<tr><td>&nbsp;</td></tr>\n";

  print "<tr><td align=right>UNIT NAME:&nbsp;\n<select name=\"unitname\" onchange=\"document.SUBMIT.submit();\">\n";

  if(param('unitname') ne "") {
    foreach $key (sort keys(%unitmap)) {
      print "<option value=\"$key\"";
        if( "$key" eq param('unitname') ) { print " selected "; }
        print ">$key\n";
    }
  } else { # no explict request
    $count=0;
    foreach $key (sort keys(%unitmap)) {
      print "<option value=\"$key\"";
        if($count++==$defaultUnitCount) { print " selected "; }
        print ">$key\n";
    }
  }

  print "</select>\n</td><td width=40>&nbsp; </td></tr>\n";

  print
  "\n<tr><td align=right>STUDENT ID:&nbsp;",
    textfield('studentid','',30,30),
    "&nbsp;</td><td width=40>&nbsp; </td></tr>",
  "\n<tr><td align=right>PASSWORD:&nbsp;",
    password_field(-name=>'password', -value=>'',-size=>30,-maxlength=>30),
    "&nbsp;</td><td width=40>&nbsp; </td></tr>\n";


  print

  "\n<tr><td align=right>",
    "<input type=button value='restart' onclick=\"location='$submitURL';\">",
    "&nbsp;",
    "<input type=reset value=reset>&nbsp;",
#    "<input type=button value='backstep' onclick=\"history.go(-1);\">\n",
    "\n&nbsp; &nbsp;",
    "<input type=button name=logout onclick=\"window.open('$submitURL');top.close();\" value=\"secure logout\">&nbsp;",
    "<input type=submit value=\"submit\"></td></tr>",


  "\n<tr><td align=right>ver 2.01 &copy; <font size=-2> 2012 ZHUHAN JIANG</font></td></tr>";


print "\n<tr><td align=right><font size=-1 face='Arial, Helvetica, sans-serif'><br>If you are returned to a page after the login,<br>just follow the links there again.</font></td></tr>";


 if($msg!~/^\s*$/) {
   if($msg=~/^<!([^\s]+)>/) {
     print "\n<tr></tr><td>&nbsp;</td><tr><td bgcolor=$1 colspan=2 align=center><font color=white>$msg</font></td></tr>\n";
   }
   else {
     print "\n<tr></tr><td>&nbsp;</td><tr><td bgcolor=red colspan=2 align=center><font color=white>$msg</font></td></tr>\n";
   }
 }

 print "</table>\n"
   . "</td></tr></table>\n"
   . "</form>\n</body></html>\n";

 $msg=param('filereceipt');
 if($msg!~/^\s*$/) {
   print "<script language=javascript>\n<!--\n",
   "document.SUBMIT.receipt.value+=\"".param('filereceipt')."\\n\";",
   "\n//-->\n</script>\n";
 }

 if(param('unitname') eq "") {
   param(-name=>'unitname',-default=>(sort keys(%unitmap))[$defaultUnitCount]);
 }
 if(param('purpose') eq "") {
   param(-name=>'purpose',-default=>(sort keys(%purposemap))[0]);
 }
}


sub escape {
my($text)=@_;
  $text=~ s/([^a-zA-Z0-9])/sprintf('%%%x',ord($1))/eg;
  return $text;
}


sub verifyPassword { # global: $passwordcrypt
my($cgi)=@_;
my($sessionid,$studentid,$password,$cookie,$c1,$c2,$c3);
  $studentid=$cgi->param('studentid');
  $password=$cgi->param('password');
  if(passwordOk($studentid,$password,$passwordcrypt)) {
#    $cookie=$cgi->cookie(-name=>'loginid', -value=>"$studentid");
#    print $cgi->header(-cookie=>$cookie, -type=>'text/html');

    $c1=$cgi->cookie(-name=>'loginid', -value=>"$studentid");
    $c2=$cgi->cookie(-name=>'unitshortname', -value=>"$unitShortName");

    $sessionid=makeCryptedTime($studentid . ' ' . $unitShortName);
    $c3=$cgi->cookie(-name=>'sessionid', -value=>"$sessionid");

    print $cgi->header(-cookie=>[$c1,$c2,$c3], -type=>'text/html');

    print "<script language=javascript>\n"
          . "location='${submitURL}?unitname="
          . escape(param('unitname')) . "';\n"
          . "</script>\n";
    return 1;
  }
  if($password!~/^\s*$/ || $studentid!~/^\s*$/) {
    param(-name=>'msg',-default=>"Invalid password for the student ID $studentid");
  }
  param(-name=>'password',-default=>'');
  return 0;
}


#
# 1. generate 4 random numbers between 0 and 51: r0, r1, r2, r3
# 2. time in seconds converted to base 52 number in 8 digits: t0, t1, .., t7
# 3. ordinal values of first 4 letters of login name applied by modulus 4: n0, n1, n2, n3
# 4. form array  ALL= (r0,r1,r2,r3, t0,t1,t2,t3, t4,t5,t6,t7, n0,n1,n2,n3)
# 5. add positionwise the MASK=(r0,r1,r2,r3, r0,r1,r2,r3, r0,r1,r2,r3, r0,r1,r2,r3)
#     i.e. ALL = ALL + MASK
# 6. keep all values of array ALL between 0 and 51 via % 52
# 7. represented each element of ALL via a letter A-Z or a-z
# 8. concatenate 16 characters into a single session text string
#

# character-wise xor the text with the mask
sub xorText {
my($text,$mask)=@_;
my($m,$n,$p);
  $m=length($text);
  $n=length($mask);
  if(!$m || !$n) {
    return $text;
  }
  if($m>$n) {
    $p=int($m/$n);
    $mask= $mask x ($p+1);
  }
  $mask=substr($mask,0,$m);
  return $text ^ $mask;
}

sub getTimeNum {
my($time)=time();
my(@num)=(0,0,0,0,0,0,0,0);
my($i);
  for $i (7,6,5,4,3,2,1,0) {
    $r=$time % 52;
    $time = int($time/52);
    $num[$i]=$r;
  }
  return @num;
}

sub text2Num {
my($text)=@_;
$text=xorText($text,$sessionSecurityMask);
my($n)=length($text);
my($i,$c,@num);
  for($i=0;$i<$n;$i++) {
    $c=ord(substr($text,$i,1));
    $c %= 52;
    $num[$i]=$c;
  }
  return @num;
}

sub randomNum {
my($total)=@_;
my(@num)=();
my($i);
  for($i=0; $i<$total; $i++) {
    $num[$i]=int(rand(52));
  }
  return @num;
}

sub makeCryptedTime {
my(@time)=getTimeNum();
my(@name)=text2Num(@_);
my(@key)=randomNum(4);
my(@all);
my($text)="";
my($i,$m,$n);
  for($i=0;$i<4;$i++) {
    if($i>$#name) {
      $name[$i]=0;
    }
  }
  @all=(@key, @time, @name);
  for($i=4;$i<16;$i++) {
     $n=$all[$i%4];
     $m=$all[$i];
     $all[$i]=($m+$n) % 52;
   }
  for($i=0; $i<16; $i++) {
    $n=$all[$i];
    if($n<26) {
      $c=chr(ord('A')+$n);
    } else {
      $c=chr(ord('a')+$n-26);
    }
    $text.= $c;
  }
  return $text;
}

sub checkCryptedTime {
my($cryptedTime,$loginText)=@_;
my(@time)=getTimeNum();
my(@name)=text2Num($loginText);
my(@all)=();
my($i,$m,$n,$total,$matched);
  for($i=0;$i<4;$i++) {
    if($i>$#name) {
      $name[$i]=0;
    }
  }
  for($i=0; $i<16; $i++) {
    $c=substr($cryptedTime,$i,1);
    $n=ord($c);
    if($n<=ord('Z')) {
      $m=$n-ord('A');
    } else {
      $m=$n-ord('a')+26;
    }
    $m %=52;
    $all[$i]=$m;
  }
  for($i=4;$i<16;$i++) {
     $n=$all[$i%4];
     $m=$all[$i];
     $all[$i]=($m-$n) % 52;
   }
  $total=0;
  $n=1;
  for($i=11;$i>=4;$i--) {
    $total+= $all[$i]*$n;
    $n*=52;
  }
  $matched=1;
  for($i=12;$i<16;$i++) {
    $matched= $matched && !($all[$i] - $name[$i-12]);
   }
   if(!$matched) { $matched=0; }
  return ($total,$matched);
}

sub sessionChecked {
my($sessionid,$loginid,$unitShortName,$sessionlimit)=@_;
my($time)=time();
my($oldTime,$matched)=checkCryptedTime($sessionid,
                        $loginid . ' ' . $unitShortName);
  if(!$matched) {
    return 0;
  }
  if($time>=$oldTime) {
    $time -= $oldTime;
  } else {
    $time = $oldTime - $time;
  }
  if(!defined($sessionlimit)) {
    $sessionlimit=36000;     # 10 hours
  }
  if($time<= $sessionlimit) {
     return 1;
  } else {
     return 0;
  }
}
