#!/usr/bin/perl
# version 1.0 Zhuhan Jiang 2011

# user configuration is in "_browse.rc"
# allow only users of tutor status to access
do "./_login-b.pl";
do "./_istutor-b.pl";

# load main configuration if not already loaded
if(!$CONFIG_LOADED) {
    do "./_conf-b.rc";
}

# function lineInFile is defined in _istutor-b.pl or _ischieftutor-b.pl
$loginid=$cgi->cookie(-name=>'loginid');
$isChiefTutor=lineInFile($loginid, $fileForChiefTutors);

# locate unit short name
$unitShortName=$cgi->cookie(-name=>'unitshortname');

# download logfile: $logfiledir only created by browse.pl
$logfile="${logfiledir}/downloads.log";

# $masterroot - all browsing starts here
$masterroot="/rusers/staff/zhuhan/Homepage/courses";
$masterroot="${webhomedir}/courses";

# which directories are allowed to access
$masterpermission="/rusers/staff/zhuhan/Homepage/courses:/rusers/staff/zhuhan/Homepage/pub";
$masterpermission="${masterroot}:${webhomedir}/pub";

# which directories are NOT allowed access
# priority over masterpermission
# $browsedisapproval="$/rusers/staff/zhuhan/Homepage/courses/feedback";

# additional directory permission to set (not used yet)
$localpermission="";

# unit-wise root/permission
$unitroot=$masterroot;
$unitpermission=$masterpermission;

# read main configurations
do "./_browse.rc";

# read overriding user configurations for specific units, if available
# but chief tutors will not be subject to this limitation
if(!$isChiefTutor) {
    do "./config/${unitShortName}_browse.rc";
}

# absorb '//' and remove trailing '/' from $masterroot
1 while $masterroot=~ s/\/\//\//g;
1 while $masterroot=~ s/\/$//g;
1 while $unitroot=~ s/\/\//\//g;
1 while $unitroot=~ s/\/$//g;


# used perl modules
# use CGI qw(:standard);


$relativepath=param('path');
$file=param('file');

# extend permission to chief tutors
if($isChiefTutor) {
    $unitroot=$masterroot;
    $unitpermission=$masterpermission;
}

# decompose path info
$root=$unitroot;
if($relativepath=~ /^\s*$/) { $relativepath=""; }
$file=absolutePath($root,$relativepath,$file);

# check for illegal characters
($validity,$combined)=validCharacters($file,$relativepath);
if(!$validity) {
  print "Content-type: text/html\n\n";
  print "Illegal characters in parameters [$combined].\n";
  exit 0;
}

# check for file permission
if(
   # always force the master permission check
   !permitted($masterpermission,$localpermission,$file)
   # then check against unit permission
   || !permitted($unitpermission,$localpermission,$file)
  ) {
  if(${file}=~ /^${root}(.*)$/) { $file="~$1"; }
  print "Content-type: text/html\n\n";
  print "Permission to [$file] denied.\n";
  exit 0;
}


($head,$savedfile)=pathAheadRelative($file);
if($savedfile=~/^\s*$/) {
    $savedfile="downloaded";
}

## alternative setting - just comment it below
# use File::Basename;
# fileparse_set_fstype("VMS");
# $savedfile=basename($file,"\..*");


print "content-type: application/octet-stream\n"
 . "Content-Disposition: inline; filename=\"${savedfile}\"\n\n";

catFile($file);


# write a log entry
if(${file}=~ /^${webhomedir}(.*)$/) { $file="~$1"; }
writeLog($loginid, $unitShortName,
    $ENV{'REMOTE_ADDR'} . ":" . $ENV{'HTTP_X_FORWARDED_FOR'},
    "file:$file");



#################
# subroutines
#################

sub writeLog {
my($uid,$sid,$ip,$message)=@_;
my($time)=time;
my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime($time);
  $year=$year % 100;
  if($year<10) { $year="0$year"; }
  if(open(LOGFILE,">>$logfile")) {
    flock(LOGFILE, 2);
    print LOGFILE "$uid $sid, $ip, $mday/",$mon+1,"/$year $hour:$min:$sec, $message\n";
    flock(LOGFILE, 8);
    close(LOGFILE);
    chmod 0600, "$logfile";
  }
}


# check for the permitted characters in form parameters
sub validCharacters {
my (@allParas)=@_;
my($onePara);
my($combined);
$combined=join(' ',@allParas);
for $onePara (@allParas) {
  if($onePara !~ /^[a-zA-Z0-9_\@\/\~\-\.\x2f\s:\x5c]*$/) {
    return (0,$combined);
  }
}
return (1,$combined);
}


# send data from a file to standard output
sub catFile {
my($file)=@_;
my($length,$data);
  if(open(INPUTFILE, "<$file")) {
    binmode INPUTFILE;
    while(1) {
      $length=read(INPUTFILE,$data,1024);
      if($length >0 ) {
	print $data;
      } else {
	last;
      }
    }
    close(INPUTFILE);
  }
}


# check for directory permission
sub permitted {
my($masterpermission,$localpermission,$dir)=@_;
  return ($dir ne "") && ($dir !~ /\.\./)

      # safty check to bar dirty trick users
      && ($dir =~ /^[a-zA-Z0-9_@\-\.\x2f\s:\x5c]*$/)
      # && ($dir =~ /^[a-zA-Z0-9_ \.\-\:\/]*$/)

      && withinPath($masterpermission,$dir)
      && !withinPath($savefiledisapproval,$dir)
      && ($localpermission eq "" || withinPath(localpermission,$dir)) ;
}


# check if dir is leading part of one of permissionpath separated by ":"
sub withinPath {
my($permissionpath,$dir)=@_;
my(@allpaths,$path);
  if("$dir" eq "") {
    return 0;
  }
  @allpaths=split(/:/, $permissionpath);
  foreach $path (@allpaths) {
    if($dir =~ m#^$path.*# ) {
      return 1;
    }
  }
  return 0;
}


# remove leading/trailing spaces, absorb "//", remove trailing "/"
sub absolutePath {
my($root,$relativepath,$dir)=@_;
my($absolute);
  if($dir=~/^\//) {
    $absolute=$dir;
  } else {
    $absolute="$root/$relativepath/$dir";
  }
  1 while $absolute=~ s/\/\//\//g;
  if($absolute=~/^(\/.*)\/$/) {
    $absolute="$1";
  }
  return $absolute;
}


sub pathAheadRelative {
my($tail)=@_;
my($head)="";
  if($tail=~/^(.*)\/([^\/]*)$/) {
    $head="$1";
    $tail="$2";
  }
  return ($head,$tail);
}

