#!/usr/bin/perl

#---------------------------------------------
# (c) Zhuhan Jiang
#     version 1.16  10 September 2009 (renamed 2.01)
# Freeware under GNU copyright convention.
#---------------------------------------------

######################
# configuration
######################

# external configuration file
$externConfig="_conf-b.rc";

# where password file and config file are kept
# $baseDir="c:/iwd2002/webquiz/";
$baseDir="./";  # must be made absolute path if not done so via $externConfig

# upload file from a remote URL
$disableUrlUpload=0;

# proxy configuration
$httpProxyUrl="http://137.154.176.10:3128";
$ftpProxyUrl=$httpProxyUrl;
$noProxyUrl="uws.edu.au";  # or "*"

# proxy username/password
$proxyUser="zjiang";
$proxyPassword="";

# realm username/password, default is for anonymous ftp
$realmUser="anonymous";
$realmPassword="";

# where to upload files (path already exists upto the last one)
$uploaddir="${baseDir}submit";

# where to keep the submission.log containing all receipts sent out
$logFileName="${uploaddir}/submission.log";
$logFileName="${logfiledir}/submission.log";

# create logfile directory if not already in existence
if(! -d "$logfiledir") { mkdir "$logfiledir",0700; }


# map unitname into a single word which is to become the prefix
# of password files;
# also works if %unitmap=();
%unitmap=(
# 'Interactive Website Development', 'IWSD',
# 'Internet Computing', 'IC',
);

# map different upload jobs into different subdirectories:
# 'full job purpose name' => 'subdir_name'
# also works if %purposemap=(); (check out validated() below )
%purposemap=(
# 'project 1' => 'proj1',
# 'project 2' => 'proj2',
);

# maximum allowed upload size in bytes
$maxsize=500*1024;

# administrator's email address
# $administratorEmail='z.jiang@scm.uws.edu.au';

# mail handler for sending receipts
$mailProgram='/usr/lib/sendmail';     # set to "" to disable

# what cryption scheme to use
# 0=use in-house cryption, 1=use unix crypt()
$passwordcrypt=1;

# ensure receipt can't be faked when perpetrator doesn't know this seed
$receiptRandomSeed="some text";

# link password file with unitname via %unitmap
$fileForPasswords="_password.txt";
$passwordfile=$baseDir.$unitmap{ param('unitname') }.$fileForPasswords;

# tuts configuration file and tuts registration file
$fileForTutsConfig="_tuts_conf.txt";
$fileForTutsRegistry="_tuts_reg.txt";
#$tutsConfigFile=$baseDir.$unitmap{ param('unitname') }.$fileForTutsConfig;
#$tutsRegFile=$baseDir.$unitmap{ param('unitname') }.$fileForTutsRegistry;

# only those registered for a tut are allowed to submit?
$tutRegForSubmit=0;

# unit/purpose/duedate data
# *[*] <UNITNAME>, <UNITDIR>
# <PROJECTNAME>, <PROJDIR>, <DUEDATE>, <LATEDATE>
# **=default unit
#
$cfgtext=<<EOF_UNITS_AND_DUEDATES

* Interactive Website Development , IWSD
########################################

IWSD: project 1 ,  proj1, 2002/05/31_00:00,  06/02,
IWSD: workshop 1 , work1, @  # @=allow submission at anytime

** Internet Computing , IC   # **=default
########################################

IC: project 2 ,  proj2,  05/31,  06/02,
IC: workshop 2 , work2,  # no duedate=submission not allowed at anytime

EOF_UNITS_AND_DUEDATES
;

# above configurations will be overriden by $externConfig
# unless you disable the external configuration by commenting out the next line


######################################
# don't change anything below
# unless you know what you are doing
######################################

# include configuration file
do "${baseDir}$externConfig";

# fetch unit/purpose/duedates info
@cfgarray=split(/\n/,$cfgtext);


sub trim {
my($s)=@_;
  if($s=~/^\s*(.*\S)\s*$/) { return "$1"; }
  return "$s";
}

$defaultUnitCount=0;   # remember the selection
$defaultUnitName="";

sub parseCfg { # global: $defaultUnitCount, $defaultUnitName
my(@cfg)=@_;
my($data)=[];
my($count,$subcount)=(0,0);
my($title,$alias)=("","");
my($aline,$name,$due,$late);
my($entry)=[];
my($titlecount)=-1;
  foreach $aline(@cfg) {
    $name=$dir=$due=$late="";
    if($aline=~/^([^#]*)#.*$/) {
      $aline="$1";
    }
    if($aline=~/^\s*$/) { next; }
    if($aline=~/^(.*)$/) { $aline="$1"; }
    $aline=~tr/a-zA-Z0-9[](){}<>+\-*\/,.:!?@#$%^&= /_/cs;
    if($aline=~/^([^,]*),(.*)$/) {
      $name="$1"; $aline="$2";
      if($aline=~/^([^,]*),(.*)$/) {
        $dir="$1";  $aline="$2";
        if($aline=~/^([^,]*),(.*)$/) {
          $due="$1";  $aline="$2";
          if($aline=~/^([^,]*),(.*)$/) {
            $late="$1";  $aline="$2";
          } else { $late=$aline; }
        } else { $due=$aline; }
      } else { $dir=$aline; }
    } else { $name=$aline; }
    $name=trim($name);
    $dir=trim($dir);
    $due=trim($due);
    $late=trim($late);
    if($name=~/^\*+\s*(.*)$/) {
      $aline="$1";
      $titlecount++;
      if($name=~/^\*\*/) {
        $defaultUnitCount=$titlecount;
        $defaultUnitName=$aline;
      }
      # $name="$1";
      $name=$aline;
      if($title ne "") {
        $data->[$count++]=[$title,$alias,$entry];
      }
      $title=$name; $alias=$dir;
      $subcount=0;
      $entry=[];
    }
    else {
      $entry->[$subcount++]=[$name,$dir,$due,$late];
    }
  }
  if($#cfg>=0) { $data->[$count++]=[$title,$alias,$entry]; }
  return $data;
}


sub cvtCfg{ # global: $defaultUnitCount
my($data,$unit)=@_;
my($i,$j,$name,$entry);
my(%unitmap)=();
my(%purposemap)=();
my(%duedate)=();
my(%latedate)=();
  $i=0;
  if(!defined($unit)) { $unit=""; }
  while(1) {
    $entry=$data->[$i];
    if(!defined($entry)) { last; }
    $name=$entry->[0];
    $unitmap{"$name"}=$entry->[1];
    if("$name" eq "$unit" || $i==$defaultUnitCount && "$unit" eq "") {
      $entry=$entry->[2];
      $j=0;
      while(1) {
        if(!defined($entry->[$j])) { last; }
        $name=$entry->[$j][0];
        $purposemap{"$name"}=$entry->[$j][1];
        $duedate{"$name"}=$entry->[$j][2];
        $latedate{"$name"}=$entry->[$j][3];
        $j++;
      }
    }
    $i++;
  }
  return (\%unitmap,\%purposemap,\%duedate,\%latedate);
}


# fetch all the unit info
@cfgParsed=parseCfg(@cfgarray);
if(param('unitname') eq "") {
  param(-name=>'unitname',-default=>$defaultUnitName);
}
(*unitmap,*purposemap,*duedate,*latedate)=cvtCfg(@cfgParsed,param('unitname'));

# link password file with unitname via %unitmap;
$passwordfile=$baseDir.$unitmap{ param('unitname') }.$fileForPasswords;

# tuts configuration and registration files
$tutsConfigFile=$baseDir.$unitmap{ param('unitname') }.$fileForTutsConfig;
$tutsRegFile=$baseDir.$unitmap{ param('unitname') }.$fileForTutsRegistry;


# command line operations for receipt verification
if($#ARGV>=0 && $ARGV[0] ne "-t" ) {
  $cmd=uc($0);
  print "$cmd -t [ uploadDir ] < receiptFile\nversion 2.01 1/1/2012 Zhuhan Jiang\n\nmake sure config.rc is available at current directory!\noutput pattern:\n*|+ FILENAME: ...\n? comparing with <UPLOADEDFILE>\n[ok, #1, #2] =: from ... [#3, #4] ...\n-> 'ok' means receipt format ok for this file entry\n-> #1=#3 and #2=#4 means this file is the correct upload\n";
  exit 0;
}

# test authenticity of a receipt
if($#ARGV>=0 && $ARGV[0] eq "-t" ) {
  authenticate($#ARGV>0? $ARGV[1]:".");
  exit 0;
}


########
# submission essentials below

use strict 'refs';
use lib '..';
use CGI qw(:standard);
use CGI::Carp qw/fatalsToBrowser/;
use LWP::UserAgent;

# URL of this submission program
$submitURL=url();

# default message (don't edit)
$donothing=" select a submission";  # leading ' ' makes this item 1st
$purposemap{$donothing}='';

# change to the upload directory
$purpose=$purposemap{ param('purpose') };

if(!chdir("$uploaddir")) {
  mkdir "$uploaddir", 0700;
  chdir "$uploaddir";
}

# make alias directory
$uploaddir.="/". $unitmap{ param('unitname') };
if(!chdir("$uploaddir")) {
  mkdir "$uploaddir", 0700;
  chdir "$uploaddir";
}

$uploaddir="$uploaddir/$purpose";
$uploaddir=~ tr/ /_/;

if(!chdir("$uploaddir")) {
  mkdir "$uploaddir", 0700;
  chdir "$uploaddir";
}


sub quickCrypt {
my($password)=@_;
my($set,$length,$salt);
  $set="ABCDEFGHIJKLMNOPQRSTUVWXYZ./0123456789";
  $set=$set.lc(substr($set,0,26));
  $length=length($set);
  $salt= substr($set,rand($length),1).substr($set,rand($length),1);
  return crypt($password,$salt);
}


sub codePassword {
my($rawpasswd)=@_;
my($a,$b,$c,$d,$i);
my(@num)=();
  for($i=0;$i<8;$i++) { $num[$i]=ord(substr("$rawpasswd",$i,1)); }
  $a=chr($num[0]);
  $b=chr($num[1]);
  for($i=1;$i<8;$i++) {
    if($num[$i]==32) {
      $num[$i]=$i+97;
      if($i==1) { $b="B"; }
    }
  }
  $c=$d=0;
  for($i=0;$i<4;$i++) {
    $c+=$num[2*$i];
    $d+=$num[2*$i+1];
  }
  return $a.$c.$b.$d;
}


sub checkPassword {
my($uid,$rawpasswd,$pfile)=@_;
my($passwd)=codePassword($rawpasswd);
my($aline,$ppasswd,$psubdir,$puid);
my($matched)=0;
  if(open(PASSFILE, "<$pfile")) {
    while(<PASSFILE>) {
      $aline=$_;
      ($ppasswd,$psubdir,$puid) = split(/,/, $aline);
      if($puid=~ /^(.*)\n$/) { $puid="$1"; }
      if($ppasswd eq $passwd  && $puid eq $uid) {
        $matched=1;
        last;
      }
    }
    close(PASSFILE);
  }
  return $matched;
}


sub checkCryptPassword {
my($uid,$rawpasswd,$pfile)=@_;
my($aline,$ppasswd,$psubdir,$puid);
my($matched)=0;
  if(open(CRYPTPASSFILE, "<$pfile")) {
    while(<CRYPTPASSFILE>) {
      $aline=$_;
      ($puid,$ppasswd,$psubdir) = split(/,/, $aline);
      if($psubdir=~ /^(.*)\n$/) { $psubdir="$1"; }
      if("$puid" eq "$uid" && crypt($rawpasswd,$ppasswd) eq "$ppasswd") {
        $matched=1;
        last;
      }
    }
    close(CRYPTPASSFILE);
  }
  return $matched;
}


sub passwordOk { # global: $passwordfile
my($uid,$password,$useCrypt)=@_;
  return $useCrypt?
    checkCryptPassword($uid,$password,$passwordfile)
    : checkPassword($uid,$password,$passwordfile);
}


sub getUserInfo {
my($uid,$pfile)=@_;
my($aline,$ppasswd,$psubdir,$puid,$pname);
my($matched)=0;
my($name)="$uid";
  if(open(BOTHPASSFILE, "<$pfile")) {
    while(<BOTHPASSFILE>) {
      $aline=$_;
      ($puid,$ppasswd,$psubdir,$pname)=split(/,/, $aline);
      if("$puid" eq "$uid") {
        $matched=1;
        last;
      }
    }
    close(BOTHPASSFILE);
  }
  if($matched) {
    $name="$pname";
    if($name=~/^\s*$/) { $name="$uid"; }
    $name=~tr/\n//;
  }
  return ($name,$psubdir);
}


sub readFile {
my($file)=@_;
my($length,$total);
my($content)="";
  if(open(CHECKSUM, "+<$file")) {
    binmode CHECKSUM;
    seek CHECKSUM,0,2;
    $length=tell(CHECKSUM);
    seek CHECKSUM,0,0;
    $total=read(CHECKSUM,$content,$length);
    close(CHECKSUM);
    if($total<$length) { $content=""; }
  }
  return $content;
}


sub checkSum {
my($chksum,$str)=@_;
my($j,$ch,$strwidth);
  $strwidth=length($str);
  for($j=0,$ch=1;$j<$strwidth;$j++) {
    $ch+= ord(substr($str,$j,1)) * (1+ ord(substr($str,$j,1)));
  }
  for($j=0;$j<$strwidth;$j++) {
    # $chksum+=(($chksum+$ch)^ ord(substr($str,$j,1))) * (1+ord(pack("c",$j)));
    $chksum+=(($ch )^ ( ord(substr($str,$j,1)))) * ($chksum % 7);
  }
  return $chksum % 1024*1024;
}


sub makeReceipt {
my($remotefile)=@_;
my($seed)=int(rand(1024*1024));
my($receipt)=checkSum($seed,readFile($remotefile));
  return "[$seed, $receipt]";
}


sub submitPage { # $defaultUnitCount: global variable
my($msg)=param('msg');
my($key,$count);

  print <<EOF_SUBMITPAGE
content-type: text/html

<html>

<script language=javascript>
<!--
function checkUrl() {
var obj=document.SUBMIT.urlfile, r=new RegExp("^https?://[^/]*.uws.edu.au\$","g");
  r=new RegExp("^/+\s*\$","g");
  if(obj.value.match(r)) {
    obj.focus();
    obj.select();
    return false;
  }
  return true;
}

function checkPurpose() {
var obj=document.SUBMIT.purpose;
  if(obj.selectedIndex==0) {
    obj.focus();
    return false;
  }
  return true;
}


function validatedFile(obj) {
var ob=obj.filename;
 if(ob.value.search(/\;/)>=0) {
    // alert('No ";" are allowed in filenames!');
    ob.focus();
    ob.select();
    return false;
  }
  return true;
}

function validated(obj) {
var ob;
  if(!obj.studentid.value.match(/[a-z0-9]/i)) {
    ob=obj.studentid;
    ob.value="ID please";
    ob.focus();
    ob.select();
    return false;
  }
  return checkPurpose(); //or just 'return true;' to disable it
}
//-->
</script>

EOF_SUBMITPAGE
;


  print "<body><form name=SUBMIT action=\"$submitURL\" method=post ENCTYPE=\"multipart/form-data\" onsubmit=\"return (validated(this)&&validatedFile(this));\">\n"
   . "<table cellpadding=30 bgcolor=" . $mainBackgroundColor . "><tr><td>\n"
   . "<input type=hidden name=mode value=''>\n"
   . "<table border=0>\n"
   . "<tr><td colspan=2 align=center><font face='Arial, Helvetica, sans-serif' size=+3><b>ONLINE SUBMISSION</b></font></td></tr>\n"
   . "<tr><td>&nbsp;</td></tr>\n";

  print "<tr><td align=right>UNIT NAME:&nbsp;\n<select name=\"unitname\" onchange=\"document.SUBMIT.submit();\">\n";

  if(param('unitname') ne "") {
    foreach $key (sort keys(%unitmap)) {
      print "<option value=\"$key\"";
        if( "$key" eq param('unitname') ) { print " selected "; }
        print ">$key\n";
    }
  } else { # no explict request
    $count=0;
    foreach $key (sort keys(%unitmap)) {
      print "<option value=\"$key\"";
        if($count++==$defaultUnitCount) { print " selected "; }
        print ">$key\n";
    }
  }

  print "</select>\n</td><td width=40>&nbsp; </td></tr>\n";

  print
  "\n<tr><td align=right>STUDENT ID:&nbsp;",
    textfield('studentid','',30,30),
    "&nbsp;</td><td width=40>&nbsp; </td></tr>",
  "\n<tr><td align=right>PASSWORD:&nbsp;",
    password_field(-name=>'password', -value=>'',-size=>30,-maxlength=>30),
    "&nbsp;</td><td width=40>&nbsp; </td></tr>",

  "\n<tr><td align=right>EMAIL:&nbsp;",

    ("$mailProgram" eq ""?
     textfield(-name=>'email', -title=>'address to email the receipt', -default=>'',-size=>30,-maxlength=>60, -onfocus=>'blur();')
    :textfield(-name=>'email', -title=>'address to email the receipt', -default=>'',-size=>30,-maxlength=>60, -onfocus=>'self.status=\'*** Must use SCIT email address only ***\';', -onblur=>'self.status=self.defaultStatus;', -XXonchange=>'if(this.value.indexOf(\'cit.uws.edu.au\')<0) alert(\'Must use SCIT email address\');')
    ),

    "&nbsp;",

   hidden(-name=>'receipt', -default=>''),
   "<input type=button title='cut-paste and save it as separate file as your receipt' value=receipts onclick=\"if(document.SUBMIT.receipt.value=='')return false;",
   "receiptHandle=window.open();",
   "receiptHandle.document.write(",
   "'<font face=Arial,Helvetica,sans-serif size=+3><b>UPLOAD RECEIPTS</b></font>'+",
   "'<pre>'+document.SUBMIT.receipt.value+'</pre>');receiptHandle.document.close();\">",
   "</td></tr>\n";

  print "<tr><td align=right>PURPOSE:&nbsp;\n<select name=\"purpose\">\n";
  foreach $key (sort keys(%purposemap)) {
    print "<option value=\"$key\"";
      if( "$key" eq param('purpose') ) { print " selected "; }
      print ">$key\n";
  }
  print "</select>";


  print
  "\n&nbsp; &nbsp; &nbsp; LATE:&nbsp;",
    checkbox(-name=>'late', -title=>'tick here for a late submission', -value=>'late', -label=>' '),
    "</td><td width=40>&nbsp; </td></tr>";

  if(!$disableUrlUpload) {

  print
  "\n<tr><td align=right>URL:&nbsp;",
    textfield(-name=>'urldir', -default=>'http://www.cit.uws.edu.au',
      -size=>35, -onblur=>''),
    "<font size=+2><b>/</b></font>",
     #don't want to remember the last name
     # textfield('urlfile','', 13),
    "<input type=text name='urlfile' value='' size=13 onblur='checkUrl();'>",
    "&nbsp;</td><td width=40>&nbsp; </td></tr>",

  "\n<tr><td align=right><font size=-2>REALM USER/PASSWORD:&nbsp;",
    textfield(-name=>'realmuser', -default=>"$realmUser", -size=>9),
    password_field('realmpassword','', 5),
    "&nbsp; &nbsp; NO PROXY:&nbsp;",
    textfield(-name=>'noproxy', -default=>"$noProxyUrl", -size=>12),
            # -onfocus=>'blur();document.SUBMIT.proxyuser.focus();'),
    "</font>&nbsp;</td><td width=40>&nbsp; </td></tr>",

  "\n<tr><td align=right><font size=-2>PROXY:&nbsp;",
    textfield(-name=>'proxyurl', -default=>"$httpProxyUrl",
    -size=>25, -onfocus=>'blur();document.SUBMIT.proxyuser.focus();'),
    "&nbsp; USER/PASSWORD:",
    textfield(-name=>'proxyuser', -default=>'', -size=>5),
    password_field('proxypassword','', 5),
    "</font>&nbsp;</td><td width=40>&nbsp; </td></tr>";
  }

  print
  "\n<tr><td align=right>FILE:&nbsp;",
    # filefield('filename','',40),  ### updated: 10/9/2009
    filefield(-name=>'filename', -size=>'40', -title=>'no special characters like ";" are allowed in the filename'),
    "</td><td width=40>&nbsp; </td></tr>",

  "\n<tr><td align=right><font size=-2>LIST UPLOADS</font>",
    checkbox(-name=>'listupload', -title=>'list all uploaded files after submission', -value=>'listupload', -label=>' '),
    "\n&nbsp; &nbsp;<font size=-2>ALLOW OVERWRITE</font>",
    checkbox(-name=>'overwrite', -title=>'allow overwriting previously uploaded files', -value=>'overwrite', -label=>' '),
    "\n&nbsp; &nbsp;<font size=-2>LOCK UPLOAD</font>",
    checkbox(-name=>'lockupload', -title=>'disabled', -value=>'lockupload', -label=>' ',
      # makes sense on unix system with CGIWRAP
      -onclick=>'document.SUBMIT.lockupload.checked=false;'),
    "</td><td width=40>&nbsp; </td></tr>",

  "\n<tr><td align=right>",
    "<input type=button value='restart' onclick=\"location='$submitURL';\">",
    "&nbsp;",
    "<input type=reset value=reset>&nbsp;",
#    "<input type=button value='backstep' onclick=\"history.go(-1);\">\n",
    "\n&nbsp; &nbsp;",
    "<input type=button name=logout onclick=\"if(self==top)window.open('$submitURL');top.close();\" value=\"secure logout\">&nbsp;",
    "<input type=submit value=\"submit\"></td></tr>",


  "\n<tr><td align=right>ver 2.01 &copy; <font size=-2> 2012 ZHUHAN JIANG</font></td></tr>";

 if($msg!~/^\s*$/) {
   if($msg=~/^<!([^\s]+)>/) {
     print "\n<tr></tr><td>&nbsp;</td><tr><td bgcolor=$1 colspan=2 align=center><font color=white>$msg</font></td></tr>\n";
   }
   else {
     print "\n<tr></tr><td>&nbsp;</td><tr><td bgcolor=red colspan=2 align=center><font color=white>$msg</font></td></tr>\n";
   }
 }


 print "</table>\n"
   . "</td></tr></table>\n"
   . "</form>\n</body></html>\n";

 $msg=param('filereceipt');
 if($msg!~/^\s*$/) {
   print "<script language=javascript>\n<!--\n",
   "document.SUBMIT.receipt.value+=\"".param('filereceipt')."\\n\";",
   "\n//-->\n</script>\n";
 }

 if(param('unitname') eq "") {
   param(-name=>'unitname',-default=>(sort keys(%unitmap))[$defaultUnitCount]);
 }
 if(param('purpose') eq "") {
   param(-name=>'purpose',-default=>(sort keys(%purposemap))[0]);
 }
}


sub timeStamp {
my $studentid="@_";
my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
  $year+=1900; $mon++;
  return "$hour:$min:$sec, $mday/$mon/$year, $studentid";
}


# outdated subroutine to deal with tuts registration in multiple files
sub grepGroupFile {
my($id,$mask,$dir)=@_;
my($names,$file,$out);
my($suffix)="";
  if($id=~/^\s*$/) { return $suffix; }
  if(opendir(DIR,"$dir")) {
    @names=readdir(DIR);
    while(<@names>) {
      $file="$_";
      if($file=~/^.*~$/) { next; }
      if($file=~ /^$mask(.*)$/) {
	  $out=`grep -w "^$id" "$dir/$file"`;
	  if($out!~/^\s*$/) {
 	    $suffix="$1";
            last;
          }
      }
    }
    closedir(DIR);
  }
  return $suffix;
}


sub getGroupName {
my($id,$tutRegFile,$tutConfFile)=@_;
my($aline);
my($tutClass)="";
my($valid)=0;
  if(open(TUTREG, "<$tutRegFile")) {
    flock(TUTREG,1);  #1=LOCK_SH
    while(<TUTREG>) {
      $aline=$_;
      # remove comments
      if($aline=~/^([^\#]*)\#.*$/) { $aline="$1";}
      if($aline=~ /^\s*(\S+)\s+(\S+).*$/) {
	  if("$id" eq "$1") {
	      $tutClass="$2";
	      last;
	  }
      }
    }
    flock(TUTREG,8);  #8=LOCK_UN
    close(TUTREG);
  }
  if(open(TUTCONF, "<$tutConfFile")) {
    flock(TUTCONF,1);  #1=LOCK_SH
    while(<TUTCONF>) {
	$aline=$_;
        # remove comments
        if($aline=~/^([^\#]*)\#.*$/) { $aline="$1";}
	if($aline=~ /^\s*(\S+)\s.*$/) {
	    if($tutClass eq "$1") {
		$valid=1;
		last;
	    }
	}
    }
    flock(TUTCONF,8);  #8=LOCK_UN
    close(TUTCONF);
  }
  if(!$valid) { $tutClass=""; }
  return $tutClass;
}


sub lineContentInfo {
my($bytes)=@_;
my($lCount,$lExtra,$wCount,$wExtra,$width)=(0,0,0,0,0);
my(@fields);
  $width=length($bytes);
  $lCount=@fields=split(/\n/,$bytes);
  if($width && substr($bytes,$width-1,1) ne "\n") {
    $lExtra=-1;
  }
  $wCount=@fields=split(/\s+/,$bytes);
  if($wCount && $fields[0] eq "") {
    $wCount--;
  }
  if($width && substr($bytes,$width-1,1) !~ /^\s+$/) {
    $wExtra=-1;
  }
  return ($lCount,$lExtra,$wCount,$wExtra,$width);
}


sub uploadFile {
my($localfile,$dir,$overwrite)=@_;
my($remotefile,@fields);
my($readCount,$uploadBuffer);
my($lCount,$lExtra,$wCount,$wExtra,$width,$lExtraLast,$wExtraLast);
my($lines,$words,$characters,$toolarge)=(0,0,0,0);
my($properfile)=$localfile;
my($msg,$receipt)=("","");
my($studentid)=param('studentid');
  if($properfile=~/^\s*$/) { return ""; }
  $properfile=~ s/\\/\//g;
  @fields=split("/",$properfile);
  $properfile=$fields[$#fields];
  $properfile=~ tr/a-zA-Z0-9\./_/cs;
  $remotefile="$dir\/$properfile";
  if(-e "$remotefile" && !$overwrite) {
    return "aborted: $remotefile already exists";
  }
  if(! -d "$dir") { mkdir $dir,0700; }
  if(!open(UPLOADFILE,">$remotefile")) {
    if($localfile !~ /^\s*$/) {
      return "denied: fail to create $remotefile for $localfile";
    }
    return "";
  }

  binmode UPLOADFILE;

  $lExtraLast=$wExtraLast=0;
  while(($readCount=read($localfile,$uploadBuffer,10240))>0) {
    if ($maxsize>$characters) {
      print UPLOADFILE $uploadBuffer;
      ($lCount,$lExtra,$wCount,$wExtra,$width)=lineContentInfo($uploadBuffer);
      $lines+=$lCount+$lExtraLast;
      $words+=$wCount;
      if(substr($uploadBuffer,0,1) !~ /^\s+$/) {
        $words+=$wExtraLast;
      }
      $characters+=$width;
      $lExtraLast=$lExtra;
      $wExtraLast=$wExtra;
    } else { $toolarge=1; }
  }

  close $localfile;
  close UPLOADFILE;

  $msg="<!blue>$properfile: $lines lines, $words words, $characters bytes";
  $receipt="=: from $properfile " . makeReceipt($remotefile) . " " . timeStamp($studentid);
  $receipt="\\n<br>$receipt ". quickCrypt($receiptRandomSeed.$receipt) . " for [" . param('unitname') ."]\\n";

  chmod 0600, $remotefile;

  if($toolarge) {
    ($lines,$words,$characters) = (0,0,0);
    unlink $remotefile;
    $msg="File $localfile too large (&gt; " . int($maxsize/1024) . "K)";
    $receipt="";
  }

  if(-z $remotefile) { unlink $remotefile; $receipt=""; }
  return ($msg,$receipt);
}


sub uploadUrlFile {
my($urldir,$urlfile,$dir,$overwrite,
      $proxyUser,$proxyPassword,$realmUser,$realmPassword)=@_;
my($ua,$req,$res,$url);
my($properfile)=$urlfile;
my(@fields,$remotefile,$status,$characters);
my($msg,$receipt)=("","");
my($proxyurl,$noproxy);
my(@noproxylist);
my($studentid)=param('studentid');

  if($urlfile=~/^\s*$/) { return ""; }
  $properfile=~ s/\\/\//g;
  @fields=split("/",$properfile);
  $properfile=$fields[$#fields];
  $remotefile="$dir\/$properfile";
  if(-e "$remotefile" && !$overwrite) {
    return "aborted: $remotefile already exists";
  }
  if(! -d "$dir") { mkdir $dir,0700; }

  $proxyurl=param('proxyurl');
  $noproxy=param('noproxy');
  $url="${urldir}/${urlfile}";
  $ua=LWP::UserAgent->new;
  $ua->max_size($maxsize+1024);  # attempting a bit more
  $ua->timeout(60);  # 1 minute
  $ua->env_proxy();
  $ua->proxy(['http', 'gopher'] => "$proxyurl");
  $ua->proxy('ftp', "$proxyurl");

  # *=no proxy for everything, BLANK=no proxy for nothing (not even localhost)
  # others='localhost' auto-added to no-proxy 'others'
  if("$noproxy"=~/^\s*$/) {
    @noproxylist=();
  } elsif(",$noproxy,"=~/,\s*\*+\s*,$/) {
    @noproxylist=('');
  } else {
    @noproxylist=split(/,/, "localhost,$noproxy");
  }
  #@noproxylist=('localhost','uws.edu.au');
  #$ua->no_proxy('localhost', "$noproxy");
  $ua->no_proxy(@noproxylist);

  $req=HTTP::Request->new(GET => "$url");
  $req->authorization_basic("$realmUser", "$realmPassword");
  $req->proxy_authorization_basic("$proxyUser", "$proxyPassword");

  $res=$ua->request($req);
  if(!$res->is_success) {
    return "fail to fetch $url";
  }

  if(!open(UPLOADURLFILE,">$remotefile")) {
    return "denied: fail to create $remotefile for $localfile";
  }
  binmode UPLOADURLFILE;

  $characters=length($res->content);
  $status=1;
  if($characters<$maxsize) {
    $status=print UPLOADURLFILE $res->content ;
  }
  $status=close(UPLOADURLFILE) && $status;
  if(!$status) {
    return "fail to write $remotefile";
  }

  $msg="<!blue>$properfile: $characters bytes";
  $receipt="=: from $url " . makeReceipt($remotefile) . " " . timeStamp($studentid);
  $receipt="\\n<br>$receipt ". quickCrypt($receiptRandomSeed.$receipt) . " for [" . param('unitname') ."]\\n";

  chmod 0600, $remotefile;

  if($characters>=$maxsize) {
    $characters=0;
    unlink $remotefile;
    $msg="File $url too large (&gt; 500K)";
    $receipt="";
  }

  if(-z $remotefile) { unlink $remotefile; $receipt=""; }
  return ($msg,$receipt);

}


sub listFiles {
my($dir)=@_;
my($list)="";
  if(opendir(DIR,"$dir")) {
    @names=readdir(DIR);
    while(<@names>) {
      if($_ eq "." || $_ eq ".." || $_ =~/^\s*$/ ) { next; }
      my($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
        $atime,$mtime,$ctime,$blksize,$blocks)=stat("$dir/$_");
      my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime($mtime);
      $year+=1900; $mon++;
      $list="$list\n<option>$_  &nbsp; - &nbsp;  $size &nbsp; - &nbsp; $hour:$min:$sec, $mday/$mon/$year</option>";
    }
    closedir(DIR);
  }
  return
  "<form><select>\n<option>** UPLOADED FILES **</option>$list\n</selct></form>";
}


sub existItem {
my($item,%items)=@_;
my($key);
  foreach $key (keys(%items)) {
    if($key eq $item) { return 1; }
  }
  return 0;
}


sub checkDuedate {
my($due,$date);
my($uselatedate)=param('late') eq 'late';
my($sec,$min,$hour,$day,$month,$year,$wday,$yday,$isdst)=localtime(time);
  if(!existItem(param('purpose'),%latedate)) { return ""; }
  if(!defined($uselatedate)) { $uselatedate=0; }
  if($uselatedate) {
    $due=$latedate{param('purpose')};
    if($due=~/^\s*$/) { $uselatedate=0; }
  }
  if(!$uselatedate) {
    $due=$duedate{param('purpose')};
  }
  if($due=~/^\s*$/) { $due="00/00_00:00"; }  # bug fix 5/3/03
  if($due!~/_/) { $due.="_24:00"; }
  $month++;
  $year+=1900;
  my(@tmp)=split(/\//,$due);
  if($#tmp==1) { $due="$year/" . $due; }
  $date= $year . "/" .
        (($month <= 9)? "0":"") . $month . "/" .
	(($day <= 9)? "0":"") . $day . "_" .
	(($hour <=9)? "0":"") . $hour . ":" .
	(($min <=9)? "0":"") . $min ;
  # expired?
  if($due ne "" && $date gt $due) {
    return "submission too late or too early"
    . "<br>for " . param('unitname') . "/" . param('purpose')
    . ".<br>Did you mean to tick the LATE box?" ;
  }
  return "";
}


sub logReceipt {
my($receipt,$title,$uploaddir)=@_;
my($email)=param('email');
  if(open(LOGFILE,">>$logFileName")) {
    flock(LOGFILE,2);  #2=LOCK_EX

    my($dir)=`pwd`;
    my($i,$j);
    my(@fields)= split("/",$dir);
    $dir="";
    for($i=$#fields-1; $i>=0, $j<=2; $i--, $j++) {
	$dir=$fields[$i]."/$dir";
    }
    $title.=" .../$dir";

    if($uploaddir=~ /^.*\/([^\/]+\/[^\/]+)$/) { $uploaddir="$1"; }
    $title.="$uploaddir";

    print LOGFILE "$title, $email\n$receipt\n";
    flock(LOGFILE,8);  #8=LOCK_UN
    close(LOGFILE);
    chmod 0600, "$logFileName";
    return 1;
  }
  return 0;
}


sub dispatcher {
my($msg)=param('msg');
my($password)=param('password');
my($studentid)=param('studentid');
my($filename)=param('filename');
my($overwrite)=(param('overwrite') eq 'overwrite');
my($listupload)=(param('listupload') eq 'listupload');
my($username,$userdir,$message,$receipt,$urlmessage,$urlreceipt);
my($tutClass)="";
  param(-name=>'filereceipt', -value=>'');
  if("$studentid" ne "") {
    ($username,$userdir)=getUserInfo($studentid,$passwordfile);
    if(!passwordOk($studentid,$password,$passwordcrypt)) {
      $msg="Invalid password for $username";
    }
    else {
      $msg=checkDuedate();
      if($msg eq "") {

      my($proxyuser,$proxypassword,$realmpassword,$tmp)
           =($proxyUser,$proxyPassword,$realmPassword,"");
      $proxyuser=param('proxyuser');
      if($proxyuser=~/^\s*$/) { $proxyuser="$proxyUser"; }
      $proxypassword=param('proxypassword');
      if($proxypassword=~/^\s*$/) { $proxypassword="$proxyPassword"; }
      $realmpassword=param('realmpassword');
      if($realmpassword=~/^\s*$/) { $realmpassword="$realmPassword"; }
      if(param('realmuser') eq "anonymous") { $realmpassword=""; }


# this outdated method is replaced by the following 2 lines
#      $uploaddir.="/../../" .  $unitmap{ param('unitname') } . grepGroupFile($studentid, $unitmap{ param('unitname') }."_group", $baseDir);

      $tutClass=getGroupName($studentid, $tutsRegFile, $tutsConfigFile);
      if($tutClass ne "") { $tutClass="_" . $tutClass; }
      $uploaddir.="/../../" .  $unitmap{ param('unitname') } . $tutClass;

      if($tutRegForSubmit && $tutClass eq "") {
	  param(-name=>'msg', -value=>"failed: must enroll in a tut/prac class before submission");
	  return;
      }

      if(!chdir("$uploaddir")) {
        mkdir "$uploaddir", 0700;
        chdir "$uploaddir";
      }
      $uploaddir="$uploaddir/$purpose";
      $uploaddir=~ tr/ /_/;
      if(!chdir("$uploaddir")) {
        mkdir "$uploaddir", 0700;
        chdir "$uploaddir";
      }


      if(!$disableUrlUpload) {
        ($urlmessage,$urlreceipt)=uploadUrlFile(param('urldir'), param('urlfile'),
           $userdir,$overwrite,
           $proxyuser,$proxypassword,
           param('realmuser'),$realmpassword);
      }

      $msg="$urlmessage";
      if($msg!~/^\s*$/) { $msg="$msg<br>"; }
      ($message,$receipt)=uploadFile($filename,$userdir,$overwrite);
      $msg="${msg}${message}";
      if($listupload) {
        if("$msg"!~/^\s*$/) { $msg="$msg<br>"; }
        $msg="$msg &nbsp;".listFiles("$userdir");
      }

      }
    }
    param(-name=>'msg', -value=>"$msg");
    $msg="";
    if($urlreceipt !~/^\s*$/) {
      $msg="* $urlmessage $urlreceipt";
      if($receipt !~/^\s*$/) { $msg="$msg<br>+ $message $receipt"; }
    } elsif($receipt !~/^\s*$/) {
      $msg="* $message $receipt";
    }
    if(($msg!~/^\s*$/)
       && !logReceipt(filterReceipt($msg),timeStamp()
          ." ($studentid," .$unitmap{ param('unitname') } .")", "$uploaddir/$userdir")) {
      return;
    }
    param(-name=>'filereceipt', -value=>"$msg");
    if("$receipt$urlreceipt" !~ /^\s*$/) {
      mailReceipt($msg,$studentid,$filename);
    }
  }
}


sub authenticate {
my($dir)=@_;
my($aline,$original,$digest,$file,$seed,$receipt);
  while(<STDIN>) {
    $aline=$_;
    if($aline=~/^[*+]\s*(\S+):\s/) { $file="$1"; }
    if($aline=~/^(=: .*)\[(\d+), (\d+)\](\s+\S+,\s+\S+,\s+\S+)\s+(\S+).*/) {
      $original="$1\[$2, $3\]$4";
      $seed=eval($2);
      $receipt=eval($3);
      $digest="$5";
      if(crypt($receiptRandomSeed.$original,$digest) eq $digest) {
        $file="$dir/$file";
        $file=~tr/\\/\//;
        $file=~s/\/\//\//g;
        print "? comparing with <$file>\n";
        $receipt=checkSum($seed,readFile($file));
        print "[ok, $seed, $receipt] $aline";
        next;
      }
    }
    print $aline;
  }
}


sub filterReceipt {
my($receipt)=@_;
  $receipt=~ s/<[^<>]*>//g;
  $receipt=~ s/\\n/\n/g;
  return $receipt;
}


sub mailReceipt {
my($receipt,$studentid,$filename)=@_;
my($email)=param('email');
  if("$mailProgram" eq "" || "$email" =~ /^\s*$/
    # || "$email" !~ /.*\.uws\.edu\.au$/  # send only local emails
    ) { return; }
  open (MAIL, "|$mailProgram -t") || die "Can't open mailprogram!\n";
  print MAIL "To: $email\n";
  print MAIL "From: $administratorEmail\n";
  print MAIL "Subject: $studentid: Receipt of File Submission for $filename\n\n";
  print MAIL "UPLOAD RECEIPTS\n\n". filterReceipt($receipt)."\n\n";
  print MAIL "Please contact $administratorEmail if there are any problems.";
  print MAIL "\nAdministrator \n";
  close (MAIL);
}



dispatcher();
submitPage();
