#!/usr/bin/perl

###### note ######
# PERL on ssh.scm.uws.edu.au behaves differently from that on staff.scm.uws.edu.au
# This script must run on staff.scm.uws.edu.au
######################


################ user configuration ######
our $uwsStaffId="2000318";
our $uwsStaffPassword="";
################ end of user configuration #####


use strict; #'vars';

require HTML::TreeBuilder;
require HTTP::Headers;
require HTTP::Request;
require HTTP::Response;
use HTTP::Request::Common;
use LWP::UserAgent;
use HTTP::Cookies;
use CGI;
use MIME::Base64;

# current site for the platform web
our $platformwebstaff="https://platformwebstaff.uws.edu.au";

our $ua = LWP::UserAgent->new;
our $cookie_jar = HTTP::Cookies->new;


##########
# debugging tool
sub display {
my($title,$response)=@_;
  print $title, $response->is_success? " SUCCEDED":" FAILED", "\n";
}


##########
# collect all hyperlinks from a page content
sub parseTags {
my($content)=@_;
my $tree=HTML::TreeBuilder->new; # empty tree
my @links=();
  $tree->parse($content);
  $tree->elementify();
  for (@{ $tree->extract_links('a') }) {
    my($link,$element,$attr,$tag)=@$_;
    $links[$#links+1]=$link;
  }
  $tree = $tree->delete;
  return @links;
}


##########
# walk into an url and get the page
sub followUrl {
my($url)=@_;
my($request,$response);
  $request=HTTP::Request->new(GET => $url);
  $cookie_jar->add_cookie_header($request);
  $response=$ua->request($request);
  if(!$response->is_success) {
    return $response;
  }
  $cookie_jar->extract_cookies($response);
  return $response;
}


##########
# walk into several absolute urls one after another
sub followUrls {
my(@urls)=@_;
my $response;
  for(my $i=0; $i<=$#urls; $i++) {
    $response=followUrl($urls[$i]);
    if(!$response->is_success) {
      return $response;
    }
  }
  return $response;
}


##########
# walk into several urls all from the same base url
# $base has the trailing "/"
sub followBaseUrls { # $base eg: "http://www.scm.uws.edu.au/~zhuhan/"
my($base,@urls)=@_;
my $response;
  for(my $i=0; $i<=$#urls; $i++) {
    $response=followUrl($base . $urls[$i]);
    if(!$response->is_success) {
      return $response;
    }
  }
  return $response;
}


##########
# remove trailing blanks
sub stripTrailingBlanks {
my($str)=@_;
  if($str=~/^(.*\S)\s*$/) {
    $str=$1;
  }
  return $str;
}


##########
# simulate the login process
sub simLogin {
my($url,$request,$response,$c)=(
  "${platformwebstaff}/pweb/pweb_staff/staffmenu.asp");
  $request=HTTP::Request->new( POST => $url);
  $request->content_type('application/x-www-form-urlencoded');
  $c=$uwsStaffPassword;
  if(!$c) { $c=decode_base64("YWRtaW5pc3RyYXRvcg=="); } # administrator
  $request->content("usertype=staff&userid=$uwsStaffId&pword=$c");
  $response=$ua->request($request);
  $cookie_jar->extract_cookies($response);
  return $response;
}


##########
# login and walk to the page containing links to the unit
sub initialise {
my($year,$period)=@_;  #eg: ("2005", "AU"
my $codeBase="${platformwebstaff}/pweb/pweb_staff";
my $response;
  $response=simLogin();
  if(!$response->is_success) { return $response; }
  $response=followBaseUrls($codeBase,
    "/adminsupport.asp",
    "/AdminTimetables.asp?yr=${year}",
    "/Admintimetables2.asp?p=${period}&open=Y"
  );
  return $response;
}


##########
# parse out paras for doPostPage()
sub parseDoPostPage {
my($url)=@_;
my($success,$subject,$period,$year,$campus,$class,$actionPage)=0;
  if( $url=~
/^javascript:doPostPage\(\'([^,]*)\',\s*\'([^,]*)\',\s*\'([^,]*)\',\s*\'([^,]*)\',\s*\'([^,]*)\',\s*\'([^,]*)\'.*\)/) {
      $subject=stripTrailingBlanks($1);
      $period=stripTrailingBlanks($2);
      $year=stripTrailingBlanks($3);
      $campus=stripTrailingBlanks($4);
      $class=stripTrailingBlanks($5);
      $actionPage=stripTrailingBlanks($6);
      $success=1;
  }
  return ($success,$subject,$period,$year,$campus,$class,$actionPage);
}


##########
# parse out paras for doPost()
sub parseDoPost {
my($url)=@_;
my($success,$tutId,$comment)=0;
  if( $url=~ /^javascript:doPost\(\'([^,]*)\',\s*\'([^,]*)\'.*\)/) {
      $tutId=stripTrailingBlanks($1);
      $comment=stripTrailingBlanks($2);
      $success=1;
  }
  return ($success,$tutId,$comment);
}


##########
# fetch the list of names for the whole lecture class;
# still works if $campus is missing
sub classList {
my($year,$period,$subject,$class,$campus)=@_;
# eg. ("2005", "AU", "300155", "CAMDAY"/"CAMEVENING")
my $codeBase="${platformwebstaff}/pweb/pweb_staff";
my ($response,$content);

  $response=initialise($year,$period);
  if(!$response->is_success) { return ""; }

  # 1st method: check all links to download the class list;
  # contains only units "owned" by the login id
  my @links=parseTags($response->content);
  for(my $i=0; $i<=$#links; $i++) {
    my $url=$links[$i];
    my($success,$vSubject,$vPeriod,$vYear,$vCampus,$vClass,$vActionPage)=
        parseDoPostPage($url);
    if( $success && ($vClass=~/^${class}/) && ($vSubject=~/^${subject}/)
        && ($vActionPage=~/^sublisttut\.asp/) ) {
      $url="${vActionPage}?sub=${vSubject}&p=${vPeriod}&year=${vYear}"
             . "&campuscode=${vCampus}&classcode=${vClass}";

      # $content.="\r\n#$url\r\n";
      $response=followBaseUrls($codeBase,
          "/$url",
          "/download.asp?period=${year}_${period}&sub=$subject"
        );
      next if !$response->is_success;
      $content.=$response->content;
    }
  }

  # 2nd method: fetch the class list directly;
  # used to be able to fetch list not owned by the login id (not anymore)
  if($content=~/^\s*$/) {
    $response=followBaseUrls($codeBase,
      "/sublisttut.asp?sub=${subject}&p=${period}&year=${year}&campuscode=${campus}&classcode=${class}",
      "/download.asp?period=${year}_${period}&sub=$subject"
    );
    if($response->is_success) {
     # $content.="\r\n#/sublisttut.asp?sub=${subject}&p=${period}&year=${year}&campuscode=${campus}&class=${class}\r\n";
      $content.=$response->content;
    }
  }
  return $content;
}


##########
# comment off lines that are not student records;
# applicable to class lists
sub filter {
my($content)=@_;
my @lines=split(/\r\n/,$content);
my($output,$i,$text);
  $output="";
  for($i=0; $i<=$#lines; $i++) {
#    if($lines[$i]!~/^s*\d+\s+.*$/) {
    if($lines[$i]!~/^s*\d+,.*$/) {
      $output .= "#";
    }
    $text=$lines[$i];
    $text=~tr/,/\t/;
    $output.=$text."\n";
  }
  return $output;
}


##########
# fetch lists of names for different tut/prac classes
sub tutList {
my($year,$period,$subject,$class,$campus)=@_;
# eg. ("2005", "AU", "300155", "CAMDAY"/"CAMEVENING")
my $codeBase="${platformwebstaff}/pweb/pweb_staff";
my ($response,$content);
  $response=initialise($year,$period);
  if(!$response->is_success) { return ""; }

  # 1st method: check all links to download the tut lists
  my @links=parseTags($response->content);
  for(my $i=0; $i<=$#links; $i++) {
    my $url=$links[$i];
    my($success,$vSubject,$vPeriod,$vYear,$vCampus,$vClass,$vActionPage)=
        parseDoPostPage($url);
    if( $success && ($vClass=~/^${class}/) && ($vSubject=~/^${subject}/)
        && ($vActionPage=~/^admintimetables3new\.asp/) ) {
      $url="${vActionPage}?sub=${vSubject}&p=${vPeriod}&year=${vYear}"
              . "&campuscode=${vCampus}&classcode=${vClass}";
      $content .= getOneTutPage("/$url", $year, $period, $subject);
    }
  }

  # 2nd method: fetch the tut lists directly;
  # used to be able to fetch lists not owned by the login id (not anymore)
  if($content=~/^\s*$/) {
      $content .= getOneTutPage(
        "/admintimetables3new.asp?subs=${subject}&p=${period}&year=${year}&campuscode=${campus}&classcode=${class}",
        $year, $period, $subject
      );
  }
  return $content;
}


##########
# go through the page for one delivery and
# for each tut/prac class link download the list of names;
# namelists for lectures are ignored
sub getOneTutPage {
my($url,$year,$period,$subject)=@_;
my $codeBase="${platformwebstaff}/pweb/pweb_staff";
my($response,$content);
  $response=followBaseUrls($codeBase, $url);
  if(!$response->is_success) { return ""; }
  my @links=parseTags($response->content);
  for(my $i=0; $i<=$#links; $i++) {
    my $url=$links[$i];
    my($success,$vID1,$vComment)=parseDoPost($url);
    if($success) {
      $url="tutdisplay1.asp?ID1=$vID1&comment=$vComment";
      my $id=$vID1;
      $response=followUrl($codeBase."/".$url);
      $response=download($codeBase."/download1.asp", "p=${period}&yr=${year}&id=$id");
      next if !$response->is_success;
      my $tmp=getType($response->content);
      if($tmp!~/^\s*$/) {
        $content.="\n#tutorial $id for unit ${subject} in ${year}_$period\n" . $tmp;
      }
      #$content.=getType($response->content);
    }
  }
  return $content;
}


##########
# post paras to an url and get its returned page
sub download {
my($url,$paras)=@_;
my($request,$response);
  $request = HTTP::Request->new( POST => $url);
  $request->content_type('application/x-www-form-urlencoded');
  $cookie_jar->add_cookie_header($request);
  $request->content($paras);
  $response = $ua->request($request);
  $cookie_jar->extract_cookies($response);
  return $response;
}


##########
# examine the page content to see if it is a list for practicals or lectures
sub getType {
my($content)=@_;
my @lines=split(/\r\n/,$content);
my($output,$name,$campus,$place);
  $output="";
  for(my $i=0; $i<=4; $i++) {
   if($lines[$i] =~/^\s*Practical\s*,\s*(\S+\s+\S+\s+\S+)\s+(\S+)\s*(.*)$/) {
     $name=$1;
     $campus=$2;
     $place=$3;
     $name=~ tr/ /_/;
     $place=~ tr/ \(\)/_/d;

     for($i++; $i<=$#lines; $i++) {
#       if($lines[$i]=~/^s*\d+\s+(\d+)\s+(.*)$/) {
       if($lines[$i]=~/^s*\d+,(\d+),(.*)$/) {
         $output .= "$1\t${campus}_${name}_$place\t$2\n";
       }
     }
     return $output;
   }
  }
  return $output;
}


##########
# process one subtask
sub dispatch {
my($mode,@paras)=@_;
  if($#paras<2) {
    return;
  }
  if($mode) {
    print filter(classList(@paras));
  } else {
    print tutList(@paras);
  }
}


##################
## MAIN
##################

if($#ARGV < 0) {
  print uc($0), " - by Zhuhan Jiang, ver 6, 13 Sept 2011\n";
  print "Useage:  $0 [-c | -t] year period subject [class [campus] ] ... \n";
  print "         -c  get class list\n";
  print "         -t  get tutorial/practical list\n";
# print "             must explicitly supply campus [CA | PA | PE] for others' units\n";
  print "e.g.\n";
  print "$0 -c 2010 AU 300580 PAREVENING PA  -t 2010 AU 300580 CAMDAY\n";
  print "$0 -c 2010 AU 300580  -t 2010 AU 300580\n";
  exit 0;
}

outer:
for(my $i=0; $i<=$#ARGV; ) {
  if( ("$ARGV[$i]" eq "-c") || ("$ARGV[$i]" eq "-t") ) {
    my $mode= "$ARGV[$i]" eq "-c" ;
    my @paras=();
    for($i++, my $j=0; ($i<=$#ARGV) && ($j<=4); $i++,$j++) {
      if($ARGV[$i]=~ /^-/) {
        dispatch($mode,@paras);
        next outer;
      }
      $paras[$j]=$ARGV[$i];
    }
    dispatch($mode,@paras);
  }
  else {
    $i++;
  }
}


#####################################
## different ways to fetch students
#
# print filter(classList("2005", "AU", "3001", "PAREVENING", "PA"));
# print filter(classList("2005", "AU", "300155", "PAREVENING"));
# print filter(classList("2005", "AU", 300155));
#
# print tutList("2005", "AU", "300155");
# print tutList("2005", "AU", "300155", "CAMDAY");
# print tutList("2005", "AU", "3001", "PAREVENING", "PA");
#####################################

#print tutList("2005", "SP", "300104", "CAMDAY", "CA");
#print filter(classList("2007", "AU", 300155));
#print tutList("2007", "AU", "300155", "CAMDAY", "CA");
