#!/usr/bin/perl

# view files within feedback subdirectory only
do "./_login-b.pl";

# load main configuration if not already loaded
if(!$CONFIG_LOADED) {
    do "./_conf-b.rc";
}

# logging of editors will not be visible
$removeLeadingComments=1;

# downloading supported via "method=download"?
$supportDownload=0;

# file viewing is currently not logged;
# download logfile: $logfiledir only created by browse.pl
$logfile="${logfiledir}/downloads.log";

# support tools for the browsing;
# need to apply escape() to all parameters
# need to apply escape() to all parameters
my $savefile_pl="savefile.pl";
my $logout_pl="logout-b.pl";


# file types for special treatment
my %doctypes= (
  'jpeg' => 'image/jpeg',
  'jpg' => 'image/jpeg',
  'gif' =>  'image/gif',
  'mpeg' => 'video/mpeg',
  'txt' => 'text/plain',
  'html' => 'text/html',

  'pdf' => 'application/pdf',
  'doc' => 'application/octet-stream',
  'docx' => 'application/octet-stream',
  'xls' => 'application/octet-stream',
  'xlsx' => 'application/octet-stream'

# 'exe' => 'application/octet-stream'
);

# disable the above document types (comment it out to enable it)
# %doctypes=();

# locate unit short name
$unitShortName=$cgi->cookie(-name=>'unitshortname');

# $masterroot - all browsing starts here
$masterroot="$webhomedir/courses/$unitShortName/feedback";

# which directories are allowed to access
$masterpermission="$masterroot";

# additional directory permission to set (not used yet)
$localpermission="";

## unit root and permission
$unitroot="${masterroot}/$unitShortName";
$unitpermission="${unitroot}";

# which directories are allowed to access
$feedbackrootdir="${unitroot}/feedback";
$feedbackpermission="${feedbackrootdir}";

# read main configurations
do "./_feedback.rc";


# used perl modules
# use CGI qw(:standard);

$currentfile=$file=param('file');
$relativepath=param('path');

# method='download' - only download it
$method=param('method');

$root=$feedbackrootdir;
if($relativepath=~ /^\s*$/) { $relativepath=""; }
$file=absolutePath($root,$relativepath,$file);


# check for illegal characters
($validity,$combined)=validCharacters($file,$relativepath);
if(!$validity) {
  print "Content-type: text/plain\n\n";
  print "Illegal characters in parameters [$combined].\n";
  exit 0;
}

# check for file permission
if(!permitted($feedbackpermission,$localpermission,$file)) {
  if(${file}=~ /^${root}(.*)$/) { $file="~$1"; }
  print "Content-type: text/html\n\n";
  print "Permission to [$file] denied.\n";
  exit 0;
}


my($head,$basename)=pathAheadRelative($file);
my $key,$readCount,$readBuffer;
my $extension=lc(getExtension($basename));

# download it/no viewing
if( $supportDownload  # direct download supported?
    && $method eq 'download') {
    print "content-type: application/octet-stream\n"
	. "Content-Disposition: inline; filename=\"$basename\"\n\n";
    catFile($file);
    exit 0;
}

# try to view/run the file
foreach $key (keys %doctypes) {
  if($extension ne "" && $key eq $extension) {
    print "content-type: ", $doctypes{$key},
       "\nContent-Disposition: inline; filename=\"$basename\"\n\n";
    if(open(DATA,"<$file")) {
      # binmode DATA;
      while(($readCount=read(DATA,$readBuffer,1024))>0) {
        print $readBuffer;
      }
      close(DATA);
      exit 0;
    }
    last;
  }
}


# treat it a raw text file
print <<EOF_PART_ONE
Content-type: text/html

<html><head><title>Read File</title></head>

<style type="text/css">
a:link {  text-decoration: none; color:0000ff;}
a:active { text-decoration: none; color:3399ff;}
a:visited { text-decoration: none; color:60a0a0;}
//a:hover { text-decoration: none; color:ffd700; color:ff0000;}
a:hover{ background-color:#80e0e0; color:black; }

.packed {
  table-layout:fixed;
  border-top: 0px solid #888888;
  border-collapse: collapse;
}

.packed td {
//  border-bottom: 1px dashed #dddddd;
  padding: 0px 0px;
  overflow:hidden;
  text-overflow:ellipsis;
  white-space:nowrap;
}

</style>

<UL>
EOF_PART_ONE
    ;



print "<H1><B><font face='Arial, Helvetica, sans-serif'>FILE: &nbsp;<!a href="
  . makeUrl($savefile_pl) . "?file=$currentfile&path=$relativepath> $basename<!/a></font></B></H1>\n";


print "<pre><hr width=75% align=left>";
if(open(DATA, "<$file")) {
  while (<DATA>) {
    $aline=$_;

    if($removeLeadingComments) {
      $removeLeadingComments=0;
      $aline="";
      if(/^\s*\#/) {
	while(<DATA>) {
          if(! /^\s*\#/) {
            $aline="$_";
            last;
          }
	}
      }
    }

    do {
      my $str=substr($aline,0,80);
      my $rest=substr($aline,80);
      $aline=$str;

      $aline=~ s/\&/\&amp\;/g;
      $aline=~ s/\"/\&quot\;/g;
      $aline=~ s/</\&lt\;/g;
      $aline=~ $/>/\&gt\;/g;
      print $aline;

      $aline=$rest;
    } until ($rest=~/^\s*$/);

  }
  close(DATA);
}
print "</pre>\n";

# html tail
print "<P><hr align=left width=75%><P>\n"
  . "<form>"
  . "<input type=button onclick='history.back();' value='back'>&nbsp; "
  . "<!input type=button onclick=\"location='". makeUrl($logout_pl)
  . "';\" value='logout browse'>"
  . "</form>\n";





#################
# subroutines
#################

sub writeLog {
my($uid,$sid,$ip,$message)=@_;
my($time)=time;
my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime($time);
  $year=$year % 100;
  if($year<10) { $year="0$year"; }
  if(open(LOGFILE,">>$logfile")) {
    flock(LOGFILE, 2);
    print LOGFILE "$uid $sid, $ip, $mday/",$mon+1,"/$year $hour:$min:$sec, $message\n";
    flock(LOGFILE, 8);
    close(LOGFILE);
    chmod 0600, "$logfile";
  }
}


# check for the permitted characters in form parameters
sub validCharacters {
my (@allParas)=@_;
my($onePara);
my($combined);
$combined=join(' ',@allParas);
for $onePara (@allParas) {
  if($onePara !~ /^[a-zA-Z0-9_\@\/\~\-\.\x2f\s:\x5c]*$/) {
    return (0,$combined);
  }
}
return (1,$combined);
}


# check for directory permission
sub permitted {
my($masterpermission,$localpermission,$dir)=@_;
  return ($dir ne "") && ($dir !~ /\.\./)

      # safty check to bar dirty trick users
      && ($dir =~ /^[a-zA-Z0-9_@\-\.\x2f\s:\x5c]*$/)
      # && ($dir =~ /^[a-zA-Z0-9_ \.\-\:\/]*$/)

      && withinPath($masterpermission,$dir)
      && !withinPath($viewfiledisapproval,$dir)
      && ($localpermission eq "" || withinPath(localpermission,$dir)) ;
}


# check if dir is leading part of one of permissionpath separated by ":"
sub withinPath {
my($permissionpath,$dir)=@_;
my(@allpaths,$path);
  if("$dir" eq "") {
    return 0;
  }
  @allpaths=split(/:/, $permissionpath);
  foreach $path (@allpaths) {
    if($dir =~ m#^$path.*# ) {
      return 1;
    }
  }
  return 0;
}


# remove leading/trailing spaces, absorb "//", remove trailing "/"
sub absolutePath {
my($root,$relativepath,$dir)=@_;
my($absolute);
  if($dir=~/^\//) {
    $absolute=$dir;
  } else {
    $absolute="$root/$relativepath/$dir";
  }
  1 while $absolute=~ s/\/\//\//g;
  if($absolute=~/^(\/.*)\/$/) {
    $absolute="$1";
  }
  return $absolute;
}


sub pathTail {
my($root,$absolute)=@_;
my($rest)="";
  if($absolute=~/^$root(.*)$/) {
    $rest="$1";
  }
  if($rest=~/^\/(.*)$/) {
    $rest="$1";
  }
  if($rest=~/^(.*)\/$/) {
    $rest="$1";
  }
  return $rest;
}


sub pathAheadRelative {
my($tail)=@_;
my($head)="";
  if($tail=~/^(.*)\/([^\/]*)$/) {
    $head="$1";
    $tail="$2";
  }
  return ($head,$tail);
}


sub makeUrl {
my($scriptname)=@_;
my($url,$relative);
  $url=url();
  $relative=url(-relative=>1);
  if($url=~/^(.*)$relative$/){
    return "$1$scriptname";
  }
  return $scriptname;
}


sub getExtension {
my($file)=@_;
  if($file=~/\.([^\/\.]*)$/) {
    return "$1";
  } else {
    return "";
  }
}


sub catFile {
my($file)=@_;
my $length;
my $data;
  if(open(INPUTFILE, "<$file")) {
    binmode INPUTFILE;
    while(1) {
      $length=read(INPUTFILE,$data,1024);
      if($length >0 ) {
	print $data;
      } else {
	last;
      }
    }
    close(INPUTFILE);
    return 1; # file opened successfully
  }
  return 0;
}
