#!/usr/bin/perl

use strict ; #'vars';
# use POSIX qw(ceil floor);
# use Clone qw(clone);
use IO::Handle;


# purpose: send and flush the message to the STDERR device
# $message	- error message to be sent to STDERR
sub printError {
my($message)=@_;
STDOUT->autoflush(1); # $|=1;
STDERR->autoflush(1);
print STDERR $message;
STDOUT->autoflush(0); # $|=0;
}


# check if an option is in @argv;
# return: 1=found; 0=not found;
sub locateCmdOption {
my($option,@argv)=@_;
my($idx);
for($idx=0; $idx<=$#argv; $idx++) {
  if($argv[$idx] eq $option) {
    return 1;
  }
}
return 0;
}


# establish the ID->name pairs
# via the password files -Pfilename
sub fetchId2Name {
my($option,@argv)=@_;
my %id2name=();
my ($cmd,$file,$line,$id,$user);
for $cmd (@argv) {
  if($cmd =~ /^$option(.*)$/) {
    $file="$1";
    if(open(PASS,"<$file")) {
      printError "# reading file: $file\n";
      while(<PASS>) {
        $line="$_";
        if($line=~ /^([^,]*),[^,]*,([^,]*),(.*)$/) {
          $id="$1";
          $user="$3 $2";
          $id2name{$id}=$user;
        }
      }
      close(PASS);
    }
  }
}
return %id2name;
}



### main program ###

if($#ARGV<0 || locateCmdOption('-h',@ARGV)) {
    print "FEEDBACK TO MARKS - ver 1, 27 April 2011 by Zhuhan Jiang\n";
    print "Useage: $0 -b -dFileDirectory -pPasswordFile feedbackFiles\n";
    print "        -b  marks to be placed within brackets\n";
    print "        -d  read files from a directory\n";
    print "        -p  names to be resolved from PasswordFile\n";
    print "        -n  new line added to separate all lines\n";
    print "e.g.  perl $0 -b -d./subdir -p./PF_password.txt > marks.txt\n";
    exit 0;
}


my $addNewLines=locateCmdOption('-n',@ARGV);
my $marksInBrackets=locateCmdOption('-b',@ARGV);
my %id2name=fetchId2Name('-p',@ARGV);
my $namesPresent;
if(scalar(keys %id2name)>0) {
  $namesPresent=1;
}
my($count,$totalItems,$markMissing,$nameMissing,$markDuplicated,$found);
my($file,$id,$comments,$marks,$line,$name);
my($fileNum,$cmdNum,$validFileNum)=(0,0,0);
$markDuplicated=$markMissing=$nameMissing=$totalItems=$count=0;

# extract files from subdirectory, place them on @argv
my @argv=@ARGV;
while(<@ARGV>) {
  if(/^-d(.*)$/) {
    my $dir=$1;
    if(opendir(DIR,$1)) {
      my @subFiles=readdir(DIR);
      # @subFiles = grep( !/^\.$/ && !/^\.\.$/ , @subFiles);
      while(<@subFiles>) {
        next if /^\.$/ || /^\.\.$/;
        push @argv, "$dir/$_";
      }
      close(DIR);
    }
  }
}

# process all the no-option files
while(<@argv>) {
  $id=$file="$_";
  if($file=~ /^-/) {
    $cmdNum++;
    next;
  }
  $fileNum++;

  # remove potential subdirectories (Unix version)
  if($id=~/\/([^\/]*$)/) {
    $id="$1";
  }

  # ID is the leftmost portion upto either '.' or '_'
  if($id=~/^([^\._]+)[\._]/) {
    $id="$1";
  }

  if(open(INPUT,"$file")) {
    $validFileNum++;
    $comments=$marks="";
    $found=0;
    while(<INPUT>) {
      $line=$_;
      if( $line=~ /\[([^\[\]]*)\s+points\s+=\s*([\+\-\d\.]*)\s+marks\]/) {
        $marks=$2;
        $comments .= "$1 points = $2 marks; ";
        $totalItems++;
        if($found) {
          $markDuplicated++;
        }
        $found=1;
      }
    }
    close(INPUT);
    if(!$found) {
      $markMissing++;
      $comments="#mark?";
    }

    $name=$id2name{$id};
    if(!defined($name)) {
      $nameMissing++;
      $name='';
      if($namesPresent) {
        $comments .= "#name?";
      }
    }

    if($marksInBrackets) {
      $marks="[$marks]";
    }
    print ++$count, "\t$id";
    if($namesPresent) {
      print "\t$name";
    }
    print "\t$marks\t$comments\n";
    if($addNewLines) {
      print "\n";
    }
  }
}

printError "\n# files containing no marks: $markMissing\n";
printError   "# files of duplicated marks: $markDuplicated\n";
if($namesPresent) {
  printError "# files of unresolved names: $nameMissing\n";
}
printError   "# cmd paras=$cmdNum, bad files=";
printError ($fileNum-$validFileNum);
printError "\n";
