#!/usr/bin/perl

%allnames=();
%allmarks=();
%alldata=();
%alltext=();

$nohalfrounding=0;
$fullmarks=0;
$rescale=1;
$directAdd=0;
$directThreshold=0;
$precision=0;
$nameWidth=0;
$nameBuff="";
@maps=();
@ostart=(); @oend=(); @nstart=(); @nend=();

if($#ARGV < 0) {
    print "Marks Rescaler - ver 1.14 April 2010 by Zhuhan Jiang\n\n";
    print "Useage:  $0 [ options ] filenames\n";
    print "         -n   display via names\n";
    print "         -m   display via marks\n";
    print "         -o   display via rank order\n";
    print "         -r   report format\n";
    print "         -t   text of comments retained\n";
    print "         -f   find the largest mark for each student\n";
    print "         -e   exact values with no half rounding\n";
    print "         -exx set output to precision of xx decimal places\n";
    print "         -wxx set width of names to xx characters\n";
    print "         -axx/yy add xx to those with yy after rescaling\n";
    print "         -xx  rescale percentage\n";
    print "         -aa:bb/cc:dd  piecewise rescaling\n";
    print " eg.     -a3/20 -50 -0:20/0:30 -20:60/30:60\n";
    print "      ** first file is special: ideal for exam marks\n";
    print "      ** first nontrivial name is adopted for each student id\n";
    print "\nFile format:\n";
    print "#...(full marks DIGITS) ... - all added up\n";
    print "  redundantDIGITS  stduentIdDIGITS Names [marks] ...\n";
    exit;
}

while (<@ARGV>) {
    if("$_" eq "-n") { $showViaNames=1; next; }
    if("$_" eq "-m") { $showViaMarks=1; next; }
    if("$_" eq "-o") { $showViaRanks=1; next; }
    if("$_" eq "-r") { $reportFormat=1; next; }
    if("$_" eq "-t") { $keepText=1; next; }
    if("$_" eq "-f") { $findLargest=1; next; }
    if("$_" eq "-e") { $nohalfrounding=1; next; }

    if("$_" =~/^-w(\d+)$/) {
	$nameWidth=eval($1);
	if($nameWidth>0) {
	    for($i=0; $i<$nameWidth; $i++) {
		$nameBuff .= " ";
	    }
	}
	next;
    }
    if("$_" =~/^-e(\d+)$/) {
	$precision=eval($1);
	next;
    }
    if("$_" =~/^-a([+\-]?\d+)\/(\d+)$/) {
	$directAdd=eval($1);
	$directThreshold=eval($2);
	next;
    }
    if("$_" =~/^-a([+\-]?\d+)$/) {
	$directAdd=eval($1);
	next;
    }

    if("$_" =~/^-(\d+\.?\d*)$/) { $rescale=1.0*$1/100; next; }

    if("$_" =~/^-(\d+):(\d+)\/(\d+):(\d+)$/) {
	$maps[$#maps+1]="$_";
	$ostart[$#maps]=$1;
	$oend[$#maps]=$2;
	$nstart[$#maps]=$3;
	$nend[$#maps]=$4;
	next;
    }

    $id="00000000";
    $marksfilename="$_";
    open(FILE, "<$_") || next ;
    while(<FILE>) {
        $line=$_;

        # get full marks from the line "#... full marks dddd ..."
        if($line=~/\s*\#.*\(\s*full marks\s+(\d*[\.\d]\d*)\s*\)/) {
	    $nextCandidate=eval($1);
	    if($findLargest) { # calculate the maximum full marks
		if($fullmarks < $nextCandidate) {
		    $fullmarks=$nextCandidate;
		}
	    } else { # add up full marks
		$fullmarks+=$nextCandidate;
	    }
	}

        elsif($line=~/^\s*\d+\s+(\d+)\s+(.*)$/) {
	    $id="$1";
	    $name=$rest="$2";
	    $mark=0;

	    if($rest=~/^(.*)\[(.*)\].*$/) {
		$name="$1";
		$mark=0+eval($2);
	    }

	    if($name=~/^(.*),\s*$/) { $name="$1"; }
	    if($name=~/^(.*\S)\s*$/) { $name="$1"; }

	    # make sure the name entry is always there
	    if(!defined($allnames{"$id"})) {
		$allnames{"$id"}=$nameBuff;
	    }
	    if(!($name =~ /^\s*$/)) {
		if($nameWidth>0) {
		    $name .= $nameBuff;
		    $name=substr($name,0,$nameWidth);
		}
		$allnames{"$id"}="$name";
	    }

	    if($findLargest) { # calculate the maximum mark
		if($allmarks{"$id"} < $mark) {
		    $allmarks{"$id"} = $mark;
		}
	    } else {  # normal adding up
		$allmarks{"$id"} += $mark;
	    }
	    $alldata{"$id"} .="+$marksfilename:$mark";
        }
	else {
	    $alltext{"$id"} .= $line;
	}
    }
    close(FILE);
}

# $rescale=0.5;
if($rescale != 1) {
    $fullmarks *=$rescale;
    foreach $key (keys(%allnames)) {
	$allmarks{$key}= $allmarks{$key}*$rescale;
    }
}

# direct add: may exceed boundary
if($directAdd != 0) {
    foreach $key (keys(%allnames)) {
	$tmp=$allmarks{$key};
	if($tmp ge $directThreshold) {
	    $allmarks{$key}= $allmarks{$key} + $directAdd;
	}
    }
}

   &mapAll();

   sub numorder { $allmarks{$b} <=> $allmarks{$a}; }
   sub strorder { $allnames{$a} cmp $allnames{$b}; }

   $count=0;
   %rawranks=();
   @allkeys= sort numorder keys(%allnames);
   foreach $key (@allkeys) {
      $rawranks{$key}=$count++;
   }

   %ranks=();
   for($i=0; $i<=$#allkeys; ) {
     $init=$i;
     $value=$allmarks{$allkeys[$i]};
     for($j=$init; $j<=$#allkeys; $j++) {
       $newvalue=$allmarks{$allkeys[$j]};
       if($newvalue != $value) { last; }
     }
     $i=$j;
     for($j=$init; $j<$i; $j++) {
       $start=$init+1;
       $ranks{$allkeys[$j]}= ($start==$i? "$start" : "$start-$i");
     }
   }

   if($showViaRanks) { &printRanks; exit 0; }

   if(!$showViaMarks) { $showViaNames=1; }
   if($showViaNames) {
      if($reportFormat) { &writeViaNames; } else { &printViaNames; }
   }
   if($showViaMarks) {
      if($reportFormat) { &writeViaMarks; } else { &printViaMarks; }
  }

sub printViaNames {
   print "\n### SORTED BY NAMES (full marks ", $fullmarks, ") ###\n\n";
   $count=0;
   foreach $key (sort strorder keys(%allnames)) {
      $count++;
      $name=$allnames{$key};
      $mark=$allmarks{$key};
      $mark=&halfround($mark);
      $data=$alldata{$key};
      if($fullmarks) {
          $percent=$mark/$fullmarks *100;
      } else {
	  $percent=0;
      }

      print "$count\t$key\t$name   [$mark]=$percent%=$data\n";
      if($keepText) {
	  $tmp=$alltext{"$key"} . "\n";
	  $tmp=~ s/\n\s*\n/\n/gs;
	  $tmp .= "\n";
	  if($tmp=~/^\s*$/) { $tmp="\n"; }
	  print $tmp;
      }
   }
}

sub printViaMarks {
   print "\n### SORTED BY MARKS (full marks ", $fullmarks, ") ###\n\n";
   $count=0;
   foreach $key (sort numorder keys(%allnames)) {
      $count++;
      $name=$allnames{$key};
      $mark=$allmarks{$key};
      $mark=&halfround($mark);
      $data=$alldata{$key};
      $percent=$mark/$fullmarks *100;

      print "$count\t$key\t$name   [$mark]=$percent%=$data\n";

      if($keepText) {
	  $tmp=$alltext{"$key"} . "\n";
	  $tmp=~ s/\n\s*\n/\n/gs;
	  $tmp .= "\n";
	  if($tmp=~/^\s*$/) { $tmp="\n"; }
	  print $tmp;
      }
   }
}

sub printRanks {
    if($fullmarks == 0) { $fullmarks=100; }
    print "### RANKING IN MARKS (id, rank, name, marks, breakdown) ###\n\n";
    foreach $key (@allkeys) {
	$count=$ranks{$key};
	$name=$allnames{$key};
	$mark=$allmarks{$key};
	$data=$alldata{$key};
	$percent=$mark/$fullmarks *100;
	print "$count\t$key\t";
        write STDOUT; # $name\t$mark\t$data\n";
	print "\n";
    }
}


format STDOUT =
@<<<<<<< @##.#%   @<<<<<<<<<<<<<< @<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$count, $percent, $name, "[$mark]", "=$data"
.


format STDOUT =
@<<<<<<< @<<<<<<<<<<   @<<<<<<<<<<<<<<<<<<<<<<<<<<<<      @<<<<<    @<<<<<< @<<<<
$count, $key, $name, $mark, $data, $rest
.


format STDOUT =
@<<<<<<< @<<<<<<<<<<   @<<<<<<<<<<<<<<<<<<<<<<<<<<<<      @<<<<<    @<<<<<<<<<<<<<<<<<<<
$count, $key, $name, $mark, $data
.


sub writeViaNames {
   print "\n### SORTED BY NAMES (full marks ", $fullmarks, ") ###\n\n";

   $count="No."; $key="Student ID"; $name="Student Name";
   $mark="Total"; $data="Exam"; $rest="Rest";
   write;
   print "--------------------------------------------------------------------------------\n";

   $count=0;
   foreach $key (sort strorder keys(%allnames)) {
      $count++;
      $name=$allnames{$key};
      $mark=$allmarks{$key};
      $mark=&halfround($mark);
      $data=$alldata{$key};
      $rest=eval($mark-$data);
      $percent=$mark/$fullmarks *100;

      $data=&pickdata(1,$data);
      write;
   }
}

sub writeViaMarks {
   print "\n### SORTED BY MARKS (full marks ", $fullmarks, ") ###\n\n";

   $count="No."; $key="Student ID"; $name="Student Name";
   $mark="Total"; $data="Exam"; $rest="Rest";
   write;
   print "--------------------------------------------------------------------------------\n";

   $count=0;
   foreach $key (sort numorder keys(%allnames)) {
      $count++;
      $name=$allnames{$key};
      $mark=$allmarks{$key};
      $mark=&halfround($mark);
      $data=$alldata{$key};
      $rest=eval($mark-$data);
      $percent=$mark/$fullmarks *100;

      $data=&pickdata(1,$data);
      write;
  }
}

sub mappedValue {
    local($value, $os, $oe, $ns, $ne)=@_;
    if($os < $oe && $value >= $os && $value <= $oe) {
	return  ( ($value-$os)*$ne + ($oe-$value)*$ns )/($oe-$os) ;
    }
    return $value;
}


sub canMap {
    local($value, $os, $oe, $ns, $ne)=@_;
    if($os < $oe && $value >= $os && $value <= $oe) {
	return 1;
    }
    return 0;
}

sub mapAll {
    local($value);
    foreach $key (keys(%allnames)) {
	for($i=0; $i<=$#maps; $i++) {
	    $value=$allmarks{$key};
	    if( &canMap($value, $ostart[$i], $oend[$i], $nstart[$i], $nend[$i])) {
         $allmarks{$key}=  &mappedValue(
		$value, $ostart[$i], $oend[$i], $nstart[$i], $nend[$i]);
         last;
     }

	}
    }
}

sub halfround {
    local($value)=@_;
    local($newvalue);
    if($nohalfrounding) { return $value; }
    if($precision>0) {
	my $a,$b;
	$a="%." . $precision ."f";
	$b=sprintf($a, $value);
	return eval($b);
    }
    $newvalue=int($value);
    if($newvalue+0.5 < $value) { $newvalue +=1.0; }
    elsif($newvalue < $value) { $newvalue += 0.5; }
    return $newvalue;
}


sub pickdata {
    local($idx, $str)=@_;
    local(@a)= split('\+', $str);
    return $a[$idx];
}
